Option Strict On
Option Explicit On

Imports Microsoft.Win32

Public Class Form1

    Private Const EffectsRoot As String = _
        "SOFTWARE\Microsoft\MediaPlayer\Objects\Effects"

    Private Structure InstWmpVis
        Dim VisName As String
        Dim ClassID As Guid
    End Structure

    Private rnd As New Random()
    Private bolRenderWin As Boolean = False
    Private cVis As New WmpVisual
    Private tRect As New WmpVisual.RECT
    Private tTimedLevel As New WmpVisual.TimedLevel
    Private bDummyFreq As Byte() = New Byte(WmpVisual.SA_BUFFER_SIZE - 1) {}
    Private bDummyWave As Byte() = New Byte(WmpVisual.SA_BUFFER_SIZE - 1) {}
    Private tInstWmpVis As InstWmpVis() = New InstWmpVis() {}

    Private Sub Form1_FormClosed(ByVal sender As Object, _
            ByVal e As System.Windows.Forms.FormClosedEventArgs) _
            Handles Me.FormClosed

        ' Timer stoppen
        Timer1.Enabled = False

        ' Ist ein Renderfenster vorhanden?
        If bolRenderWin = True Then
            ' Renderfenster lschen
            If cVis.Destroy = True Then
                bolRenderWin = False
            End If
        End If

        ' Visualisierung entladen
        cVis.DestroyVis()

        ' Klasse
        cVis = Nothing

    End Sub

    Private Sub Form1_Load(ByVal sender As Object, _
            ByVal e As System.EventArgs) Handles Me.Load

        cmdPropPage.Enabled = False
        cmdPropPage.Text = "Show Propertypage"
        cmdPresetDown.Enabled = False
        cmdPresetDown.Text = "<< Preset"
        cmdPresetUp.Enabled = False
        cmdPresetUp.Text = "Preset >>"

        ' RegistryKey, wo die Visualisierungen aufgelistet sind, ffnen
        Using VisRoot As RegistryKey = _
                Registry.LocalMachine.OpenSubKey(EffectsRoot)

            ' alle SubKeys auslesen
            Dim VisSubKeyNames As String() = VisRoot.GetSubKeyNames()

            ' Anzahl der SubKeys auslesen
            Dim VisCount As Integer = VisRoot.SubKeyCount

            ' Array zur Aufnahme der Werte dimensionieren
            Array.Resize(tInstWmpVis, VisCount)

            ' alle Visualisierungen auslesen
            For VisItem As Integer = 0 To VisCount - 1

                ' Name der Visualisierung auslesen
                tInstWmpVis(VisItem).VisName = VisSubKeyNames(VisItem)

                ' Name in die ComboBox einfgen
                cbInstVis.Items.Add(VisSubKeyNames(VisItem))

                ' RegistryKey, wo die ClassID der Visualisierungen ist, ffnen
                Using VisProp As RegistryKey = _
                    Registry.LocalMachine.OpenSubKey( _
                        EffectsRoot & "\" & VisSubKeyNames(VisItem) & _
                        "\Properties")

                    ' ClassID auslesen
                    tInstWmpVis(VisItem).ClassID = _
                        New Guid(VisProp.GetValue("classid").ToString)
                End Using
            Next
        End Using

        ' Erste Visualisierung selektieren
        cbInstVis.SelectedIndex = 0
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, _
            ByVal e As System.EventArgs) Handles Timer1.Tick

        ' Nur symbolisch ein paar Zufallsdaten generieren die in der 
        '  Visualisierung angezeigt werden sollen. Normalerweise wrden die 
        '  Daten vom WMP kommen. Hier mssen wir uns aber selbst um das befllen
        '  der Daten kmmern. Zum Beispiel knnte man sich die Daten per 
        '  waveInAddBuffer von der Soundkarte holen.
        rnd.NextBytes(bDummyFreq)
        rnd.NextBytes(bDummyWave)

        ' Genaue Informationen welche Daten in der Struktur TimedLevel, speziell
        '  fr Frequency und Waveform, erwartet werden, findet man in der 
        '  MSDN unter
        ' http://msdn.microsoft.com/en-us/library/dd564504%28VS.85%29.aspx

        ' TimedLevel Struktur fllen
        With tTimedLevel
            .State = WmpVisual.PlayerState.Play_State
            .Frequency.Data0 = bDummyFreq
            .Frequency.Data1 = bDummyFreq
            .Waveform.Data0 = bDummyWave
            .Waveform.Data1 = bDummyWave
        End With

        ' Timer stoppen damit die Visualisierung Zeit bekommt die Daten 
        '  zu rendern
        Timer1.Enabled = False

        ' ist ein Renderfenster vorhanden (IWmpEffects2)
        If bolRenderWin = True Then
            ' Visualisierung mit IWmpEffects2 auf dem Fenster rendern
            If cVis.RenderWindowed(tTimedLevel, True) = True Then
                ' war das Rendern erfolgreich, dann Timer wieder starten
                ' um die nchsten Daten zu rendern
                Timer1.Enabled = True
            End If
        Else
            ' ansonsten mit IWmpEffects rendern

            ' Rect Struktur fllen
            ' Gre und Position der Renderausgabe
            With tRect
                .Left = 0
                .Top = 0
                .Right = pbRenderVis.Size.Width
                .Bottom = pbRenderVis.Size.Height
            End With

            ' Visualisierung mit IWmpEffects auf dem Fenster rendern
            If cVis.Render(tTimedLevel, pbRenderVis.Handle, tRect) = True Then
                ' war das Rendern erfolgreich, dann Timer wieder starten
                ' um die nchsten Daten zu rendern
                Timer1.Enabled = True
            End If
        End If
    End Sub

    Private Sub cmdPropPage_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles cmdPropPage.Click

        ' Einstellungsfenster der Visualisierung anzeigen
        cVis.DisplayPropertyPage(Me.Handle)
    End Sub

    Private Sub cmdPresetUp_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles cmdPresetUp.Click

        ' ist die Nummer des aktuellen Preset kleiner der vorhandenen Presets
        If cVis.GetCurrentPreset < cVis.GetPresetCount - 1 Then
            ' Button aktivieren
            cmdPresetDown.Enabled = True

            ' nchsten Preset anzeigen
            If cVis.SetCurrentPreset(cVis.GetCurrentPreset + 1) = True Then
                ' ist die Nummer des aktuellen Preset gleich der 
                '  vorhandenen Presets
                If cVis.GetCurrentPreset = cVis.GetPresetCount - 1 Then
                    ' Button deaktivieren
                    cmdPresetUp.Enabled = False
                End If
            End If
        Else
            ' Button aktivieren
            cmdPresetUp.Enabled = True
        End If

        ' Anzeigen des aktuellen Presetnamen und die Anzahl der 
        '  vorhandenen Presets
        SetLblPresetName()
    End Sub

    Private Sub cmdPresetDown_Click(ByVal sender As System.Object, _
        ByVal e As System.EventArgs) Handles cmdPresetDown.Click

        ' ist die aktuelle Presetnummer grer 0
        If cVis.GetCurrentPreset > 0 Then
            ' Button aktivieren
            cmdPresetUp.Enabled = True

            ' vorhergehenden Preset anzeigen
            If cVis.SetCurrentPreset(cVis.GetCurrentPreset - 1) = True Then
                ' ist die aktuelle Presetnummer gleich 0
                If cVis.GetCurrentPreset = 0 Then
                    ' Button deaktivieren
                    cmdPresetDown.Enabled = False
                End If
            End If
        Else
            ' Button aktivieren
            cmdPresetDown.Enabled = True
        End If

        ' Anzeigen des aktuellen Presetnamen und die Anzahl der 
        '  vorhandenen Presets
        SetLblPresetName()
    End Sub

    Private Sub SetLblPresetName()
        ' Anzeigen des aktuellen Presetnamen und die Anzahl der 
        '  vorhandenen Presets
        lblPresetName.Text = cVis.GetPresetTitle(cVis.GetCurrentPreset) & _
            " [Preset " & (cVis.GetCurrentPreset + 1).ToString & _
            " von " & cVis.GetPresetCount.ToString & "]"
    End Sub

    Private Sub cbInstVis_SelectedIndexChanged(ByVal sender As Object, _
        ByVal e As System.EventArgs) Handles cbInstVis.SelectedIndexChanged

        ' Timer stoppen
        Timer1.Enabled = False

        ' ist ein Renderfenster vorhnden
        If bolRenderWin = True Then
            ' Renderfenster lschen
            If cVis.Destroy = True Then
                bolRenderWin = False
            End If
        End If

        ' Visualisierung entladen
        cVis.DestroyVis()

        cmdPropPage.Enabled = False
        cmdPresetDown.Enabled = False
        cmdPresetUp.Enabled = False

        ' Visualisierung initialisieren
        ' Was nicht geht sind die Visualisierungen aus der wmploc.dll.
        If cVis.InitVis( _
                tInstWmpVis(cbInstVis.SelectedIndex).ClassID) = True Then

            ' Titel der Visualisierung auslesen
            Me.Text = cVis.GetTitle

            ' Ein Visualisierungsobjekt kann mehrere unterschiedliche 
            '  Visualisierungen enthalten. Diese werden Presets genannt.

            ' sind mehrere Presets vorhanden
            If cVis.GetPresetCount - 1 > 0 Then
                ' dann den Button PresetUp aktivieren
                cmdPresetUp.Enabled = True
            End If

            ' besitzt diese Visualisierung ein Einstellungsfenster
            If (cVis.GetCapabilities And _
                    WmpVisual.Capabilities.EFFECT_HASPROPERTYPAGE) = _
                    WmpVisual.Capabilities.EFFECT_HASPROPERTYPAGE Then

                ' dann Button PropPage aktivieren
                cmdPropPage.Enabled = True
            End If

            ' Anzeigen des aktuellen Presetnamen und die Anzahl der 
            '  vorhandenen Presets
            SetLblPresetName()

            ' untersttzt die Visualisierung das Interface IWmpEffects2
            If cVis.UsedIWmpEffects2 = True Then
                ' dann verwenden wir auch die Mglichkeiten dieses Interfaces

                ' ein Renderfenster erstellen
                bolRenderWin = cVis.Create(pbRenderVis.Handle)
            End If

            ' Timer starten
            Timer1.Interval = 25
            Timer1.Enabled = True
        Else
            pbRenderVis.Invalidate()
            lblPresetName.Text = _
                "Diese Visualisierung lsst sich nicht anzeigen."
            Me.Text = "Error"
        End If
    End Sub

End Class
