Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class WmpVisual

    Public Const SA_BUFFER_SIZE As Integer = 1024
    Private Const TimedLevelMaxFrequency As Single = 22050
    Private Const TimedLevelMinFrequency As Single = 20
    Private Const S_OK As Integer = 0
    Private Const CLSCTX_INPROC_SERVER As Integer = 1
    Private IID_IUnknown As New Guid("00000000-0000-0000-C000-000000000046")
    Private WmpVis As Object = Nothing ' Visualisierungs-Objekt
    Private Effects As IWmpEffects ' Interface IWmpEffects
    Private Effects2 As IWmpEffects2 ' Interface IWmpEffects2

    <DllImport("ole32.dll", EntryPoint:="CoCreateInstance")> _
    Private Shared Function CoCreateInstance(ByRef clsid As Guid, _
        <MarshalAs(UnmanagedType.IUnknown)> ByVal inner As Object, _
        ByVal context As Integer, ByRef uuid As Guid, _
        <MarshalAs(UnmanagedType.IUnknown)> _
            ByRef rReturnedComObject As Object) As Integer
    End Function

    <DllImport("user32.dll", EntryPoint:="GetDC")> _
    Private Shared Function GetDC(ByVal hwnd As IntPtr) As IntPtr
    End Function

    <DllImport("user32.dll", EntryPoint:="ReleaseDC")> _
    Private Shared Function ReleaseDC(ByVal hwnd As IntPtr, _
            ByVal hdc As IntPtr) As Integer
    End Function

    ''' <summary>
    ''' bergabewerte fr dem Member State in der Struktur TimedLevel
    ''' </summary>
    ''' <remarks>
    '''   http://msdn.microsoft.com/en-us/library/dd564012%28VS.85%29.aspx
    ''' </remarks>
    Public Enum PlayerState As Integer
        Stop_State = 0
        Pause_State = 1
        Play_State = 2
    End Enum

    ''' <summary>
    ''' Rckgabewerte der Funktion GetCapabilities
    ''' </summary>
    ''' <remarks>
    '''   http://msdn.microsoft.com/en-us/library/dd563262%28VS.85%29.aspx
    ''' </remarks>
    Public Enum Capabilities As Integer
        EFFECT_CANGOFULLSCREEN = &H1
        EFFECT_HASPROPERTYPAGE = &H2
        EFFECT_VARIABLEFREQSTEP = &H4
        EFFECT_WINDOWED_ONLY = &H8
        EFFECT2_FULLSCREENEXCLUSIVE = &H10
    End Enum

    ''' <summary>
    ''' Wird bentigt um die Position und Gre des Visualisierungsfensters 
    ''' festzulegen, z.B. fr die Funktion Render
    ''' </summary>
    <StructLayout(LayoutKind.Sequential)> _
    Public Structure RECT
        Public Left As Integer
        Public Top As Integer
        Public Right As Integer
        Public Bottom As Integer
    End Structure

    ''' <summary>
    ''' Hilfsstruktur fr TimedLevel als Ersatz fr die zweidimensionalen Arrays
    ''' In VBC wrde die Struktur TimedLevel so auschauen
    ''' Type TimedLevel
    '''     Frequency(1, SA_BUFFER_SIZE - 1) As Byte
    '''     Waveform(1, SA_BUFFER_SIZE - 1) As Byte
    '''     State As Long ' Enum PlayerState
    '''     TimeStamp As Currency ' Type LARGE_INTEGER
    ''' End Type
    ''' </summary>
    <StructLayout(LayoutKind.Sequential)> _
    Public Structure Data
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=SA_BUFFER_SIZE)> _
        Public Data0() As Byte
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=SA_BUFFER_SIZE)> _
        Public Data1() As Byte
    End Structure

    ''' <summary>
    ''' Nimmt die Daten auf die dann gerendert werden sollen.
    ''' </summary>
    ''' <remarks>
    '''  http://msdn.microsoft.com/en-us/library/dd564504%28VS.85%29.aspx
    ''' </remarks>
    <StructLayout(LayoutKind.Sequential)> _
    Public Structure TimedLevel
        Public Frequency As Data
        Public Waveform As Data
        Public State As PlayerState
        Public TimeStamp As Int64
    End Structure

    ''' <summary>
    ''' Interface IWmpEffects
    ''' </summary>
    ''' <remarks>
    '''  http://msdn.microsoft.com/en-us/library/dd563253%28VS.85%29.aspx
    ''' </remarks>
    <Guid("D3984C13-C3CB-48e2-8BE5-5168340B4F35")> _
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IWmpEffects

        Function Render(ByRef pLevels As TimedLevel, ByVal Hdc As IntPtr, _
                ByRef pRC As RECT) As Integer
        Function MediaInfo(ByVal lChannelCount As Integer, _
                ByVal lSampleRate As Integer, _
                ByVal bstrTitle As String) As Integer
        Function GetCapabilities(ByRef pdwCapabilities As Integer) As Integer
        Function GetTitle(ByRef bstrTitle As String) As Integer
        Function GetPresetTitle(ByVal nPreset As Integer, _
                ByRef bstrPresetTitle As String) As Integer
        Function GetPresetCount(ByRef count As Integer) As Integer
        Function SetCurrentPreset(ByVal currentpreset As Integer) As Integer
        Function GetCurrentPreset(ByRef currentpreset As Integer) As Integer
        Function DisplayPropertyPage(ByVal hwndOwner As IntPtr) As Integer
        Function GoFullScreen(ByVal fFullscreen As Boolean) As Integer
        Function RenderFullScreen(ByRef pLevels As TimedLevel) As Integer

    End Interface

    ''' <summary>
    ''' Interface IWmpEffects2
    ''' </summary>
    ''' <remarks>
    '''  http://msdn.microsoft.com/en-us/library/dd563254%28VS.85%29.aspx
    ''' </remarks> 
    <Guid("695386EC-AA3C-4618-A5E1-DD9A8B987632")> _
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IWmpEffects2

        Inherits IWmpEffects
        Shadows Function Render(ByRef pLevels As TimedLevel, _
            ByVal Hdc As IntPtr, ByRef pRC As RECT) As Integer
        Shadows Function MediaInfo(ByVal lChannelCount As Integer, _
            ByVal lSampleRate As Integer, ByVal bstrTitle As String) As Integer
        Shadows Function GetCapabilities( _
            ByRef pdwCapabilities As Integer) As Integer
        Shadows Function GetTitle(ByRef bstrTitle As String) As Integer
        Shadows Function GetPresetTitle(ByVal nPreset As Integer, _
            ByRef bstrPresetTitle As String) As Integer
        Shadows Function GetPresetCount(ByRef count As Integer) As Integer
        Shadows Function SetCurrentPreset( _
            ByVal currentpreset As Integer) As Integer
        Shadows Function GetCurrentPreset( _
            ByRef currentpreset As Integer) As Integer
        Shadows Function DisplayPropertyPage( _
            ByVal hwndOwner As IntPtr) As Integer
        Shadows Function GoFullScreen(ByVal fFullscreen As Boolean) As Integer
        Shadows Function RenderFullScreen( _
            ByRef pLevels As TimedLevel) As Integer
        Function SetCore(ByVal pPlayer As IntPtr) As Integer
        Function Create(ByVal hwndParent As IntPtr) As Integer
        Function Destroy() As Integer
        Function NotifyNewMedia(ByVal pMedia As IntPtr) As Integer
        Function OnWindowMessage(ByVal Msg As Integer, _
            ByVal WParam As Integer, ByVal LParam As Integer, _
            ByRef plResultParam As Integer) As Integer
        Function RenderWindowed(ByRef pData As TimedLevel, _
            ByVal fRequiredRender As Boolean) As Integer

    End Interface

    ''' <summary>
    ''' Ldt und initialisiert eine Visualisierung
    ''' </summary>
    ''' <param name="VisGUID">GUID einer Visualisierung</param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    Public Function InitVis(ByVal VisGUID As Guid) As Boolean

        If WmpVis Is Nothing Then

            ' Visualisierung erstellen
            If CoCreateInstance(VisGUID, Nothing, _
                CLSCTX_INPROC_SERVER, IID_IUnknown, WmpVis) = S_OK Then

                ' Versuch auf IWmpEffects zu casten. Bei einem Fehlschlag,
                ' wird Nothing zurck gegeben und das Objekt WmpVis
                ' untersttzt das Interface IWmpEffects nicht.
                Effects = TryCast(WmpVis, IWmpEffects)

                If Effects IsNot Nothing Then

                    ' Versuch auf IWmpEffects2 zu casten. Bei einem Fehlschlag,
                    ' wird Nothing zurck gegeben und das Objekt WmpVis
                    ' untersttzt das Interface IWmpEffects2 nicht.
                    Effects2 = TryCast(WmpVis, IWmpEffects2)

                    ' wird das Interface IWmpEffects2 untersttzt, brauchen 
                    ' wir das Interface IWmpEffects nicht mehr.
                    If Effects2 IsNot Nothing Then
                        Effects = Nothing
                    End If

                    ' Erstes Preset setzen. Das muss sein da es ansonsten bei
                    ' einigen Visualisierungen zu einer AccessViolationException
                    ' fhrt.
                    Me.SetCurrentPreset(0)

                    InitVis = True

                End If
            End If
        End If

    End Function

    ''' <summary>
    ''' mit InitVis initialisierte Visualisierung lschen
    ''' </summary>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    Public Function DestroyVis() As Boolean

        ' ist ein Visualisierungs-Objekt vorhanden
        If WmpVis IsNot Nothing Then

            ' dann freigeben
            If Marshal.FinalReleaseComObject(WmpVis) = 0 Then

                If Effects2 IsNot Nothing Then
                    Effects2 = Nothing
                End If

                If Effects IsNot Nothing Then
                    Effects = Nothing
                End If

                WmpVis = Nothing
                DestroyVis = True

            End If
        End If

    End Function

    ''' <summary>
    ''' Ermittelt ob die Visualisierung das Interface IWmpEffects2 untersttzt
    ''' </summary>
    ''' <returns>
    '''   True = wird untersttzt / False = wird nicht untersttzt
    ''' </returns>
    Public Function UsedIWmpEffects2() As Boolean

        If Effects2 IsNot Nothing Then
            UsedIWmpEffects2 = True
        End If

    End Function

    ''' <summary>
    ''' Rendert die Visualisierung auf ein Fenster
    ''' </summary>
    ''' <param name="pLevels">Struktur TimedLevel</param>
    ''' <param name="hwndParent">Handle eines Fensters</param>
    ''' <param name="pRC">Struktur RECT</param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function Render(ByRef pLevels As TimedLevel, _
            ByVal hwndParent As IntPtr, _
            ByRef pRC As RECT) As Boolean

        ' einen DC vom Handle holen
        Dim dcParent As IntPtr = GetDC(hwndParent)

        ' ist ein DC vorhanden
        If dcParent <> IntPtr.Zero Then

            ' wenn nicht das Interface IWmpEffects2 untersttzt wird
            If Effects2 Is Nothing Then

                ' dann Funktion aus IWmpEffects verwenden
                If Effects.Render(pLevels, dcParent, pRC) = S_OK Then
                    Render = True
                End If
            Else

                ' wenn doch, dann Funktion aus IWmpEffects2 verwenden
                If Effects2.Render(pLevels, dcParent, pRC) = S_OK Then
                    Render = True
                End If
            End If

            ' DC freigeben
            ReleaseDC(hwndParent, dcParent)
        End If

    End Function

    ''' <summary>
    ''' Sendet die Anzahl der Kanle,Samplerate und einen Text an die 
    '''  Visualisierung
    ''' </summary>
    ''' <param name="lChannelCount">
    '''  Anzahl der Kanle(1 fr Mono, 2 fr Stereo)
    ''' </param>
    ''' <param name="lSampleRate">
    '''  Samplerate in Hertz (Hz). Bsp. 22500 = 22.5KHz
    ''' </param>
    ''' <param name="bstrTitle">einen Text</param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function MediaInfo(ByVal lChannelCount As Integer, _
        ByVal lSampleRate As Integer, ByVal bstrTitle As String) As Boolean

        If Effects2 Is Nothing Then
            If Effects.MediaInfo(lChannelCount, lSampleRate, _
                    bstrTitle) = S_OK Then

                MediaInfo = True
            End If
        Else
            If Effects2.MediaInfo(lChannelCount, lSampleRate, _
                    bstrTitle) = S_OK Then

                MediaInfo = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Eigenschaften der Visualisierung auslesen
    ''' </summary>
    ''' <returns>Enum Capabilities</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function GetCapabilities() As Integer

        Dim intRet As Integer = -1

        If Effects2 Is Nothing Then
            If Effects.GetCapabilities(intRet) = S_OK Then
                GetCapabilities = intRet
            End If
        Else
            If Effects2.GetCapabilities(intRet) = S_OK Then
                GetCapabilities = intRet
            End If
        End If

    End Function

    ''' <summary>
    ''' Namen der Visualisierung auslesen
    ''' </summary>
    ''' <returns>Name der Visualisierung</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function GetTitle() As String

        Dim strRet As String = ""

        If Effects2 Is Nothing Then
            If Effects.GetTitle(strRet) = S_OK Then
                GetTitle = strRet
            Else
                GetTitle = ""
            End If
        Else
            If Effects2.GetTitle(strRet) = S_OK Then
                GetTitle = strRet
            Else
                GetTitle = ""
            End If
        End If

    End Function

    ''' <summary>
    ''' Namen des aktuellen Presets auslesen
    ''' </summary>
    ''' <param name="nPreset">Presetnummer</param>
    ''' <returns>Name des Preset</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function GetPresetTitle(ByVal nPreset As Integer) As String

        Dim strRet As String = ""

        If Effects2 Is Nothing Then
            If Effects.GetPresetTitle(nPreset, strRet) = S_OK Then
                GetPresetTitle = strRet
            Else
                GetPresetTitle = ""
            End If
        Else
            If Effects2.GetPresetTitle(nPreset, strRet) = S_OK Then
                GetPresetTitle = strRet
            Else
                GetPresetTitle = ""
            End If
        End If

    End Function

    ''' <summary>
    ''' Anzahl der vorhandenen Presets in der Visualisierung auslesen
    ''' </summary>
    ''' <returns>Anzahl der vorhandenen Presets</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function GetPresetCount() As Integer

        Dim intRet As Integer = -1

        If Effects2 Is Nothing Then
            If Effects.GetPresetCount(intRet) = S_OK Then
                GetPresetCount = intRet
            End If
        Else
            If Effects2.GetPresetCount(intRet) = S_OK Then
                GetPresetCount = intRet
            End If
        End If

    End Function

    ''' <summary>
    ''' Setzt den Preset einer Visualisierung
    ''' </summary>
    ''' <param name="currentpreset">Presetnummer</param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function SetCurrentPreset(ByVal currentpreset As Integer) As Boolean

        If Effects2 Is Nothing Then
            If Effects.SetCurrentPreset(currentpreset) = S_OK Then
                SetCurrentPreset = True
            End If
        Else
            If Effects2.SetCurrentPreset(currentpreset) = S_OK Then
                SetCurrentPreset = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Aktuelle Presetnummer der Visualisierung auslesen
    ''' </summary>
    ''' <returns>aktuelle Presetnummer</returns>
    ''' <remarks>IWMPEffects</remarks>
    Public Function GetCurrentPreset() As Integer

        Dim intRet As Integer = -1

        If Effects2 Is Nothing Then
            If Effects.GetCurrentPreset(intRet) = S_OK Then
                GetCurrentPreset = intRet
            End If
        Else
            If Effects2.GetCurrentPreset(intRet) = S_OK Then
                GetCurrentPreset = intRet
            End If
        End If

    End Function

    ''' <summary>
    ''' ffnet das Einstellungsfenster der Visualisierung wenn vorhanden.
    ''' </summary>
    ''' <param name="hwndOwner">Handle eines Fensters</param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function DisplayPropertyPage(ByVal hwndOwner As IntPtr) As Boolean

        If Effects2 Is Nothing Then
            If Effects.DisplayPropertyPage(hwndOwner) = S_OK Then
                DisplayPropertyPage = True
            End If
        Else
            If Effects2.DisplayPropertyPage(hwndOwner) = S_OK Then
                DisplayPropertyPage = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Schaltet die Visualisierung in den Vollbildmodus wenn untersttzt
    ''' </summary>
    ''' <param name="fFullscreen">
    '''  True = Vollbildmodus / False = Fenstermodus
    ''' </param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function GoFullScreen(ByVal fFullscreen As Boolean) As Boolean

        If Effects2 Is Nothing Then
            If Effects.GoFullScreen(fFullscreen) = S_OK Then
                GoFullScreen = True
            End If
        Else
            If Effects2.GoFullScreen(fFullscreen) = S_OK Then
                GoFullScreen = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Rendert die Visualisierung im Vollbildmodus
    ''' </summary>
    ''' <param name="pLevels">Struktur TimedLevel</param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects</remarks>
    Public Function RenderFullScreen(ByRef pLevels As TimedLevel) As Boolean

        If Effects2 Is Nothing Then
            If Effects.RenderFullScreen(pLevels) = S_OK Then
                RenderFullScreen = True
            End If
        Else
            If Effects2.RenderFullScreen(pLevels) = S_OK Then
                RenderFullScreen = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Ermglicht den Zugriff auf die Windows Media Player APIs
    ''' </summary>
    ''' <param name="pPlayer">
    '''  normalerweise ein Pointer auf ein IWMPCore Interface
    ''' </param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects2</remarks>
    Public Function SetCore(ByVal pPlayer As IntPtr) As Boolean

        ' nur wenn die Visualisierung das Interface IWmpEffects2 untersttzt
        If Effects2 IsNot Nothing Then
            If Effects2.SetCore(pPlayer) = S_OK Then
                SetCore = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Erstellt ein Visualisierungsfenster
    ''' </summary>
    ''' <param name="hwndParent">
    '''  Handle eines Fensters, auf dem die Visualisierung
    '''   angezeigt werden soll. 
    ''' </param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects2</remarks>
    Public Function Create(ByVal hwndParent As IntPtr) As Boolean

        If Effects2 IsNot Nothing Then
            If Effects2.Create(hwndParent) = S_OK Then
                Create = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Das Visualisierungsfenster, das mit Create erstellt wurde, lschen.
    ''' </summary>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects2</remarks>
    Public Function Destroy() As Boolean

        If Effects2 IsNot Nothing Then
            If Effects2.Destroy = S_OK Then
                Destroy = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Informiert die Visualisierung das ein neues Media-Objekt geladen wurde
    ''' </summary>
    ''' <param name="pMedia">
    '''  normalerweise einen Pointer auf ein IWMPMedia Interface
    ''' </param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects2</remarks>
    Public Function NotifyNewMedia(ByVal pMedia As IntPtr) As Boolean

        If Effects2 IsNot Nothing Then
            If Effects2.NotifyNewMedia(pMedia) = S_OK Then
                NotifyNewMedia = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Sendet Nachrichten an das Visualisierungsfenster
    ''' </summary>
    ''' <param name="Msg">Message</param>
    ''' <param name="WParam">WParam</param>
    ''' <param name="LParam">LParam</param>
    ''' <returns>Rckgabewert der Funktion</returns>
    ''' <remarks>IWmpEffects2</remarks>
    Public Function OnWindowMessage(ByVal Msg As Integer, _
            ByVal WParam As Integer, ByVal LParam As Integer) As Integer

        Dim intRet As New Integer

        If Effects2 IsNot Nothing Then
            If Effects2.OnWindowMessage(Msg, WParam, LParam, intRet) = S_OK Then
                OnWindowMessage = intRet
            End If
        End If

    End Function

    ''' <summary>
    ''' Rendert eine Windowed Visualisierung
    ''' </summary>
    ''' <param name="pData">Struktur TimedLevel</param>
    ''' <param name="fRequiredRender">
    '''  True = muss sich selbst neu zeichnen / 
    '''   False = muss sich nicht selbst neu zeichnen
    ''' </param>
    ''' <returns>True = erfolgreich / False = nicht erfolgreich</returns>
    ''' <remarks>IWmpEffects2</remarks>
    Public Function RenderWindowed(ByRef pData As TimedLevel, _
            ByVal fRequiredRender As Boolean) As Boolean

        If Effects2 IsNot Nothing Then
            If Effects2.RenderWindowed(pData, fRequiredRender) = S_OK Then
                RenderWindowed = True
            End If
        End If

    End Function

    ''' <summary>
    ''' Umrechnung einer Frequenz zu einem Index fr den Member Frequency 
    '''  in der Struktur TimedLefel
    ''' </summary>
    ''' <param name="Freq">Frequenz: min 20 Hz / max 22050 Hz</param>
    ''' <returns>
    '''  Index fr den Member Frequency in der Struktur TimedLefel
    ''' </returns>
    Public Function Frequency_Index(ByVal Freq As Single) As Integer

        Frequency_Index = CInt((Freq - TimedLevelMinFrequency) / (( _
            TimedLevelMaxFrequency - TimedLevelMinFrequency) / _
            SA_BUFFER_SIZE))

    End Function

End Class
